@@ -126,6 +126,12 @@ static Bool IsSWSolidFillFaster(struct PVR2DPixmap *pdst, PVR2DBLTINFO * pBlt)
 	int sw_time = pixels >> 5;
 	int pages;
 
+	/* pixmap owned by GPU and sw_time + flush_time < hw_time */
+	if (QueryBlitsComplete(pdst, 0) != PVR2D_OK) {
+		/* surface is busy with SGX, use hardware rendering */
+		return FALSE;
+	}
+
 	if (sw_time > hw_time) {
 		if (pdst->owner == PVR2D_OWNER_GPU) {
 			/* pixmap owned by GPU, hw_time < sw_time */
@@ -152,12 +158,6 @@ static Bool IsSWSolidFillFaster(struct PVR2DPixmap *pdst, PVR2DBLTINFO * pBlt)
 		}
 	}
 
-	/* pixmap owned by GPU and sw_time + flush_time < hw_time */
-	if (QueryBlitsComplete(pdst, 0) != PVR2D_OK) {
-		/* surface is busy with SGX, use hardware rendering */
-		return FALSE;
-	}
-
 	return TRUE;
 }
 
@@ -334,32 +334,6 @@ static void PVR2DDoneSolid(PixmapPtr pDstPixmap)
 {
 }
 
-/* IsOverlapping
- * Are rectangles [[x1, y1], (x2, y2)] and
- * [[i1, j1], (i2, j2)] overlapping
- */
-static Bool IsOverlapping(int x1, int y1, int x2, int y2,
-		int i1, int j1, int i2, int j2)
-{
-	if ((x1 <= i1) && (i1 < x2)) {
-		/* top left corner */
-		if ((y1 <= j1) && (j1 < y2))
-			return TRUE;
-		/* bottom left corner */
-		if ((y1 <= j2) && (j2 < y2))
-			return TRUE;
-	}
-	if ((x1 <= i2) && (i2 < x2)) {
-		/* top right corner */
-		if ((y1 <= j1) && (j1 < y2))
-			return TRUE;
-		/* bottom right corner */
-		if ((y1 <= j2) && (j2 < y2))
-			return TRUE;
-	}
-	return FALSE;
-}
-
 /* Heuristics for choosing between software and hardware copy.
  * The heuristics will choose the solution that will take less CPU time
  * returns	TRUE  : Software solid fill is faster
@@ -368,15 +342,6 @@ static Bool IsOverlapping(int x1, int y1, int x2, int y2,
 static Bool IsSWCopyFaster(struct PVR2DPixmap *psrc, struct PVR2DPixmap *pdst,
 			   PVR2DBLTINFO * pBlt)
 {
-	/* copying overlapped rectangles with SW causes some corruptions,
-	 * visible mostly when scrolling.
-	 */
-	if ((pBlt->pDstMemInfo == pBlt->pSrcMemInfo) &&
-			IsOverlapping(pBlt->SrcX, pBlt->SrcY,
-				pBlt->SrcX + pBlt->SizeX, pBlt->SrcY + pBlt->SizeY,
-				pBlt->DstX, pBlt->DstY,
-				pBlt->DstX + pBlt->DSizeX, pBlt->DstY + pBlt->DSizeY))
-		return FALSE;
 	/* the values below are taken blindly from SolidFill heuristics */
 	/* flushing a page takes ~31 usec, we want to avoid cache flushing, so give a bigger penalty */
 	const int flush_penalty = 40;
@@ -387,6 +352,12 @@ static Bool IsSWCopyFaster(struct PVR2DPixmap *psrc, struct PVR2DPixmap *pdst,
 	int sw_time = pixels >> 5;
 	int pages_src, pages_dst;
 
+	if ((QueryBlitsComplete(pdst, 0) != PVR2D_OK)
+	    || (QueryBlitsComplete(psrc, 0) != PVR2D_OK)) {
+		/* surface is busy with SGX, use hardware rendering */
+		return FALSE;
+	}
+
 	pages_dst = (PVR2DGetFlushSize(pdst) + 4096 - 1) >> 12;
 	pages_src = (PVR2DGetFlushSize(psrc) + 4096 - 1) >> 12;
 	if (pdst->owner == PVR2D_OWNER_CPU)
@@ -402,12 +373,6 @@ static Bool IsSWCopyFaster(struct PVR2DPixmap *psrc, struct PVR2DPixmap *pdst,
 		return FALSE;
 	}
 
-	if ((QueryBlitsComplete(pdst, 0) != PVR2D_OK)
-	    || (QueryBlitsComplete(psrc, 0) != PVR2D_OK)) {
-		/* surface is busy with SGX, use hardware rendering */
-		return FALSE;
-	}
-
 	return TRUE;
 }
 
@@ -923,11 +888,11 @@ static void *PVR2DCreatePixmap2(ScreenPtr pScreen, int width, int height,
 #if USE_SHM
 		ppix->shmaddr = pSysMemInfo->pBase;
 #endif
-		ppix->screen = 1;
+		ppix->screen = TRUE;
 
 		CreateScreenPixmap = FALSE;
 	} else {
-		ppix->screen = 0;
+		ppix->screen = FALSE;
 	}
 	if (pixmapsHT)
 		x_hash_table_insert(pixmapsHT, ppix, ppix);
@@ -1020,7 +985,7 @@ static Bool PVR2DModifyPixmapHeader(PixmapPtr pPixmap, int width, int height,
 	    || (ppix->pvr2dmem == pSysMemInfo && !pPixData)) {
 
 		DestroyPVR2DMemory(ppix);
-		ppix->screen = 0;
+		ppix->screen = FALSE;
 		ppix->pvr2dmem = NULL;
 #if USE_SHM
 #if USE_MALLOC
@@ -1061,7 +1026,7 @@ static Bool PVR2DModifyPixmapHeader(PixmapPtr pPixmap, int width, int height,
 			}
 		} else if (pPixData == (void *)~0UL) {
 			ppix->pvr2dmem = pSysMemInfo;
-			ppix->screen = 1;
+			ppix->screen = TRUE;
 #if USE_SHM
 			ppix->shmid = -1;
 			ppix->shmaddr = pSysMemInfo->pBase;
